Class: Merkle::BinaryTree
- Inherits:
-
AbstractTree
- Object
- AbstractTree
- Merkle::BinaryTree
- Defined in:
- lib/merkle/binary_tree.rb
Overview
Merkle trees take the same approach as Bitcoin to construct a complete binary tree. If the number of hashes in the list at a given level is odd, the last one is duplicated before computing the next level (which is unusual in Merkle trees). So keep in mind that this following merkle tree algorithm has a serious flaw related to duplicate elements, resulting in a vulnerability (CVE-2012-2459).
Constant Summary
Constants included from Util
Util::HASH_SIZE, Util::MAX_DEPTH
Instance Attribute Summary
Attributes inherited from AbstractTree
Method Summary
Methods inherited from AbstractTree
#compute_root, from_elements, #generate_proof, #initialize
Methods included from Util
#bin_to_hex, #combine_sorted, #decode_hash, #hex_string?, #node_hash?, #normalize_hash
Constructor Details
This class inherits a constructor from Merkle::AbstractTree